# STL container: `std::pair`

`std::pair` is a simple but powerful utility in C++ that is used to store two heterogeneous objects (values) together. The objects can be of any type, and they are stored as members named first and second. `std::pair` is particularly useful when you need to return two values from a function, store key-value pairs in containers like `std::map`, or combine two related pieces of data.

## Key Characteristics of `std::pair`

- Heterogeneous Storage: A `std::pair` can store two values of different types.
- Member Access: The two values are accessed through the first and second members.
- Comparison Operators: `std::pair` supports comparison operators, allowing pairs to be compared lexicographically.
- Ease of Use: `std::pair` is easy to use and integrates seamlessly with many parts of the C++ Standard Library, such as `std::map`, `std::set`, and `std::tuple`.

## Basic Usage of std::pair

Let's explore some basic examples to understand how std::pair works.
Example 1: Creating and Using `std::pair`
```cpp
#include <iostream>
#include <utility>  // For std::pair

int main() {
    // Create a pair to store an integer and a string
    std::pair<int, std::string> myPair(1, "One");

    // Access the elements of the pair
    std::cout << "First: " << myPair.first << std::endl;
    std::cout << "Second: " << myPair.second << std::endl;

    return 0;
}
```

### Explanation:

- Creation: The pair `myPair` is created with an integer 1 as the first element and a string "One" as the second element.
- Member Access: The elements of the pair are accessed using the first and second members.

## Example 2: Using `std::make_pair`

C++ provides the utility function `std::make_pair` to create pairs more conveniently without specifying the types explicitly.
```cpp
#include <iostream>
#include <utility>

int main() {
    // Create a pair using std::make_pair
    auto myPair = std::make_pair(2, "Two");

    // Access the elements of the pair
    std::cout << "First: " << myPair.first << std::endl;
    std::cout << "Second: " << myPair.second << std::endl;

    return 0;
}
```

### Explanation:

- `std::make_pair`: Automatically deduces the types of the elements in the pair, making it easier to create pairs without specifying the types explicitly.
- Auto Type Deduction: The auto keyword is used to deduce the type of myPair.

## Example 3: Returning Multiple Values from a Function

One of the common uses of std::pair is to return multiple values from a function.
```cpp
#include <iostream>
#include <utility>

std::pair<int, int> getMinMax(int a, int b) {
    if (a < b)
        return std::make_pair(a, b);
    else
        return std::make_pair(b, a);
}

int main() {
    auto result = getMinMax(10, 20);

    std::cout << "Min: " << result.first << std::endl;
    std::cout << "Max: " << result.second << std::endl;

    return 0;
}
```

### Explanation:

- Returning Multiple Values: The function getMinMax returns a std::pair<int, int> containing the minimum and maximum of two integers.
- Structured Binding: The returned pair is stored in result, and its elements are accessed using first and second.

## Example 4: Using std::pair in a Container

std::pair is often used in associative containers like std::map to store key-value pairs.
```cpp
#include <iostream>
#include <map>

int main() {
    std::map<int, std::string> myMap;

    // Insert key-value pairs using std::pair
    myMap.insert(std::make_pair(1, "One"));
    myMap.insert(std::make_pair(2, "Two"));

    // Iterate over the map
    for (const auto& pair : myMap) {
        std::cout << "Key: " << pair.first << ", Value: " << pair.second << std::endl;
    }

    return 0;
}
```

### Explanation:

- std::pair in std::map: Each element in a std::map is a std::pair, where the first member is the key, and the second member is the value.
- Insertion: Elements are inserted into the map using std::make_pair.
- Iteration: The map is iterated over, and each key-value pair is accessed using first and second.
- Comparison of std::pair Objects
- std::pair supports comparison operators, which compare pairs lexicographically by first comparing the first members, and if they are equal, comparing the second members.

## Example 5: Comparing std::pair Objects

```cpp
#include <iostream>
#include <utility>

int main() {
    std::pair<int, std::string> pair1(1, "Apple");
    std::pair<int, std::string> pair2(2, "Banana");

    if (pair1 < pair2) {
        std::cout << "pair1 is less than pair2" << std::endl;
    }

    return 0;
}
```

### Explanation:

- Lexicographical Comparison: The comparison `pair1 < pair2` first compares the first members (`1 < 2`), so `pair1` is considered less than `pair2.`
- Operator Support: `std::pair` supports all relational operators `(==, !=, <, >, <=, >=)`.
- Use Cases for `std::pair`
- Returning Multiple Values: When you need to return two related values from a function, `std::pair` is a straightforward solution.
- Associative Containers: `std::pair` is the fundamental building block of associative containers like `std::map` and `std::multimap,` where it is used to store key-value pairs.
- Temporary Grouping: When you need to temporarily group two values together for processing or storage, `std::pair` provides a lightweight and efficient solution.

## Summary

- `std::pair:` A simple container that holds two values of possibly different types, accessed via first and second.
- Flexibility: `std::pair` is flexible and can be used to group any two types of data together.
- Utility: `std::make_pair` provides an easy way to create `std::pair` objects without explicitly specifying the types.
- Comparison: `std::pair` objects can be compared using standard comparison operators, with lexicographical ordering.

`std::pair` is a fundamental and versatile part of the C++ Standard Library, offering an easy way to store and work with pairs of related values. Its integration with containers like std::map makes it an indispensable tool in many C++ applications.
